from rich.console import Console
from rich.progress import BarColumn, MofNCompleteColumn, Progress, TextColumn
from rich.table import Column
from rich.traceback import install
import logging
from rich.logging import RichHandler

log = logging.getLogger("rich")
console = Console()
progress = Progress(
    TextColumn(
        "[progress.description]{task.description}",
        table_column=Column(width=50),
    ),
    BarColumn(bar_width=None),
    MofNCompleteColumn(table_column=Column(width=10, justify="right")),
    console=console,
    expand=True,
    transient=True,
)


def setup_logging():
    FORMAT = "%(message)s"
    logging.basicConfig(
        level="NOTSET",
        format=FORMAT,
        datefmt="[%X]",
        handlers=[RichHandler(console=console)],
    )
    install()



import json
import os
import re
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, Union

import requests
from bs4 import BeautifulSoup
from requests.utils import cookiejar_from_dict, dict_from_cookiejar


@dataclass
class CourseInfo:
    id: int
    name: str
    is_hidden: bool
    is_favourite: bool


@dataclass
class AssignmentInfo:
    id: int
    name: str
    course_id: int
    course_name: str
    due_date: datetime
    is_overdue: bool


@dataclass
class AttachmentInfo:
    filename: str
    url: str


@dataclass
class AdditionalAssignmentInfo:
    attachments: List[AttachmentInfo]
    description: str
    grade: Optional[int]
    grade_date: Optional[datetime]
    checker: Optional[str]
    feedback_comments: Optional[str]
    feedback_files: Optional[List[AttachmentInfo]]


@dataclass
class GradeInfo:
    assignment_id: int
    assignment_name: str
    grade: Optional[Union[float, str]]
    """The minimal grade possible for the assignment"""
    grade_min: Optional[Union[int, str]]
    """The maximal grade possible for the assignment"""
    grade_max: Optional[Union[int, str]]
    """The average grade of the assignment submissions"""
    grade_average: Optional[Union[float, str]]
    feedback: str


@dataclass
class RecordingInfo:
    name: str
    url: str


def try_float(s: str):
    try:
        return float(s)
    except ValueError:
        return s


def try_int(s: str):
    try:
        return int(s)
    except ValueError:
        return s


class MoodleException(Exception):
    def __init__(self, errorcode: str, message: str, link: str, more_info_url: str):
        Exception.__init__(self, errorcode)

        self.errorcode = errorcode
        self.message = message
        self.link = link
        self.more_info_url = more_info_url


class Moodle:
    SAML_RESPONSE_REGEX = re.compile(r'name="SAMLResponse" value="(.*?)"')
    SESSKEY_REGEX = re.compile(
        r'"https:\/\/moodle\.tau\.ac\.il\/login\/logout\.php\?sesskey=(.*?)"'
    )
    USER_ID_REGEX = re.compile(
        r'"https:\/\/moodle\.tau\.ac\.il\/user\/profile\.php\?id=(.*?)"'
    )

    sesskey: str
    user_id: int

    def __init__(
        self, username: str, id: str, password: str, session_file: Optional[str] = None
    ):
        self.username = username
        self.id = id
        self.password = password
        self.session = requests.Session()
        self.session_file = session_file

        if session_file is not None and os.path.exists(session_file):
            with open(session_file, "r") as f:
                session_info = json.load(f)

            if (
                "sesskey" in session_info
                and "cookies" in session_info
                and "user_id" in session_info
            ):
                self.session.cookies = cookiejar_from_dict(session_info["cookies"])
                self.sesskey = session_info["sesskey"]
                self.user_id = session_info["user_id"]
                return

        self._sign_in()

    def _sign_in(self):
        self.session = requests.Session()
        self.session.get("https://moodle.tau.ac.il/login/index.php")
        self.session.post(
            "https://nidp.tau.ac.il/nidp/saml2/sso?id=10&sid=0&option=credential"
        )
        self.session.post(
            "https://nidp.tau.ac.il/nidp/saml2/sso?sid=0",
            {
                "option": "credential",
                "Ecom_User_ID": self.username,
                "Ecom_User_Pid": self.id,
                "Ecom_Password": self.password,
            },
        )
        response = self.session.get("https://nidp.tau.ac.il/nidp/saml2/sso?sid=0")
        saml_response = Moodle.SAML_RESPONSE_REGEX.findall(response.text)[0]
        response = self.session.post(
            "https://moodle.tau.ac.il/auth/saml2/sp/saml2-acs.php/moodle.tau.ac.il",
            {
                "SAMLResponse": saml_response,
                "RelayState": "https://moodle.tau.ac.il/login/index.php",
            },
        )
        self.sesskey = Moodle.SESSKEY_REGEX.findall(response.text)[0]
        self.user_id = int(Moodle.USER_ID_REGEX.findall(response.text)[0])

        if self.session_file is not None:
            with open(self.session_file, "w") as f:
                json.dump(
                    {
                        "cookies": dict_from_cookiejar(self.session.cookies),
                        "sesskey": self.sesskey,
                        "user_id": self.user_id,
                    },
                    f,
                )

    def request_service(self, service_name: str, payload):
        result = self.session.post(
            f"https://moodle.tau.ac.il/lib/ajax/service.php?&sesskey={self.sesskey}&info={service_name}",
            json=payload,
        ).json()[0]

        if "error" in result and result["error"]:
            exception = result["exception"]

            if exception["errorcode"] == "servicerequireslogin":
                self._sign_in()
                return self.request_service(service_name, payload)

            raise MoodleException(
                exception["errorcode"],
                exception["message"],
                exception["link"],
                exception["moreinfourl"],
            )

        return result["data"]

    def get_page(
        self, page_id: Optional[str] = None, page_url: Optional[str] = None
    ) -> BeautifulSoup:
        if page_id is None and page_url is None:
            raise Exception(
                "At least one of `page_id` or `page_url` must be specified!"
            )
        if page_url is None:
            page_url = f"https://moodle.tau.ac.il/course/view.php?id={page_id}"
        result = self.session.get(page_url)
        with open("out.html", "w") as f:
            f.write(result.text)
        return BeautifulSoup(result.text, "html.parser")

    def get_courses(self, only_visible=True) -> List[CourseInfo]:
        response = self.request_service(
            "block_mycourses_get_enrolled_courses_by_timeline_classification",
            [
                {
                    "index": 0,
                    "methodname": "block_mycourses_get_enrolled_courses_by_timeline_classification",
                    "args": {
                        "offset": 0,
                        "limit": 0,
                        "classification": "all",
                        "sort": "fullname",
                        "customfieldname": "",
                        "customfieldvalue": "",
                        "groupmetacourses": 1,
                    },
                }
            ],
        )

        return [
            CourseInfo(
                info["id"], info["fullname"], info["hidden"], info["isfavourite"]
            )
            for info in response["courses"]
            if not only_visible or info["visible"]
        ]

    def get_assignments(
        self, limit=50, since=0, until=100000000000
    ) -> List[AssignmentInfo]:
        # TODO: understand the limittononsuspendedevents argument
        response = self.request_service(
            "block_timeline_extra_local_get_action_events_by_timesort",
            [
                {
                    "index": 0,
                    "methodname": "block_timeline_extra_local_get_action_events_by_timesort",
                    "args": {
                        "limitnum": limit,
                        "timesortfrom": since,
                        "timesortto": until,
                        "limittononsuspendedevents": True,
                    },
                }
            ],
        )

        return [
            AssignmentInfo(
                info["instance"],
                info["name"],
                info["course"]["id"],
                info["course"]["fullname"],
                datetime.fromtimestamp(info["timesort"]),
                info["overdue"],
            )
            for info in response["events"]
        ]

    def get_additional_info(self, assignment_id: int):
        # TODO: parse for grade, grade_date, checker, feedback_comments and feedback_files. Note that a lot of different subsets of these optionals are possible.

        page = BeautifulSoup(
            self.session.get(
                f"https://moodle.tau.ac.il/mod/assign/view.php?id={assignment_id}"
            ).text,
            "html.parser",
        )

        attachments = [
            AttachmentInfo(a.text, a["href"])
            for a in page.find_all("a")
            if "href" in a.attrs and "introattachment" in a["href"]
        ]

        description = page.find("div", {"class": "activity-description"})

        return AdditionalAssignmentInfo(
            attachments,
            str(description) if description.text.strip() != "" else "",
            None,
            None,
            None,
            None,
            None,
        )

    def download_url(self, url: str):
        response = self.session.get(url)
        return response.content

    def get_recordings(self, course_id: int):
        response = self.session.post(
            "https://moodle.tau.ac.il/blocks/panopto/panopto_content.php",
            {"sesskey": self.sesskey, "courseid": course_id},
        )
        response = BeautifulSoup(response.text, "html.parser")

        return [
            RecordingInfo(a.text, a["href"])
            for a in response.find_all("a")
            # filtering is needed to ignore the settings link
            if "Viewer.aspx" in a["href"]
        ]

    def get_grades(self, course_id: int) -> List[GradeInfo]:
        response = self.session.get(
            f"https://moodle.tau.ac.il/course/user.php?mode=grade&id={course_id}&user={self.user_id}"
        )
        response = BeautifulSoup(response.text, "html.parser")

        grades = []
        for tr in response.find_all("tr"):
            tds: List[BeautifulSoup] = tr.find_all("td")

            if len(tds) != 6:
                continue

            a: BeautifulSoup = tds[0].find("a")

            if a is None:
                continue

            grade_range = tds[3].text.split("–")

            grades.append(
                GradeInfo(
                    assignment_id=int(a["href"].split("=")[-1]),
                    assignment_name=a.text,
                    grade=None if tds[2].text == "-" else try_float(tds[2].text),
                    grade_min=try_int(grade_range[0]),
                    grade_max=try_int(grade_range[1]),
                    grade_average=None
                    if tds[4].text == "-"
                    else try_float(tds[4].text),
                    feedback=tds[5].text.strip(),
                )
            )

        return grades



import datetime
import time
from dataclasses import dataclass
from typing import Any, Dict, List, Optional
from urllib.parse import urlencode

import requests
from bs4 import BeautifulSoup

IMS_BASE_URL = "https://iims.tau.ac.il"


@dataclass(unsafe_hash=True)
class GradeInfo:
    semester: str
    """The full semester of the course, in our year format, e.g. 2025a"""

    course_id: str
    """The 8-digit ID of the course, e.g. 03661111"""

    grade: int
    """The grade of the user in the course"""

    is_exempt: bool
    """Is the student exempt from this course?"""


class IMS:
    def __init__(self, username: str, id: str, password: str):
        self.username = username
        self.id = id
        self.password = password
        self.session = requests.Session()

        self._sign_in()

    def _sign_in(self):
        self.session = requests.Session()

        # Because of stupid redirection stuff, we need to sign in to the regular stuff before IMS.
        self.session.get(IMS_BASE_URL + "/Tal/")
        self.session.post(
            "https://nidp.tau.ac.il/nidp/saml2/sso?id=10&sid=0&option=credential",
        )
        self.session.post(
            "https://nidp.tau.ac.il/nidp/saml2/sso?sid=0",
            {
                "option": "credential",
                "Ecom_User_ID": self.username,
                "Ecom_User_Pid": self.id,
                "Ecom_Password": self.password,
            },
        )
        self.session.get("https://nidp.tau.ac.il/nidp/saml2/sso?sid=0")

        response = self.session.post(
            IMS_BASE_URL + "/Tal/Login_Chk.aspx",
            data={
                "txtUser": self.username,
                "txtId": self.id,
                "txtPass": self.password,
                "enter": "כניסה",
                "javas": "1",
                "src": "",
            },
        )

        if "/Tal/Sys/Main.aspx" not in response.url:
            raise Exception("Invalid username or password!")

    def request_page(
        self,
        method: str,
        path: str,
        params: Dict[str, str],
        data: Optional[Any] = None,
        attempt_number=1,
        max_attempts=5,
        delay=0.1,
    ):
        params["id"] = self.id
        params["dt"] = datetime.datetime.now().strftime("%d%m%Y%H%M%S")

        url = IMS_BASE_URL + "/Tal/" + path + "?" + urlencode(params)
        response = self.session.request(method, url, data=data)
        result = BeautifulSoup(response.text, "html.parser")

        # The system is currently sometimes inconsistent.
        if attempt_number < 10 and (
            result.find("title") is None
            or result.find("title").text == 'אוניברסיטת ת"א - יציאה'
        ):
            time.sleep(attempt_number * delay)
            return self.request_page(
                method,
                path,
                params,
                data,
                attempt_number + 1,
                max_attempts,
                delay,
            )

        return result

    def get_study_plan_ids(self) -> List[str]:
        page = self.request_page(
            "get", "TP/Tziunim_P.aspx", params={"src": "", "sys": "tal", "rightmj": 1}
        )
        form_element = page.find("form", {"name": "frmfree"})
        return [
            input_element["value"]
            for input_element in form_element.find_all("input", {"name": "tckey"})
        ]

    def get_grades(self, study_plan_id: str, years: List[int]) -> List[GradeInfo]:
        results = []
        first_page = self.request_page(
            "post",
            "TP/Tziunim_L.aspx",
            params={"src": "", "sys": "tal", "rightmj": 1, "first": "yes", "lang": ""},
            data={
                "tckey": study_plan_id,
                "btnishur.x": 59,
                "btnishur.y": 15,
                "caller": "tziunim_p",
                "peula": 3,
                "javas": 1,
                "h_eng": "",
            },
        )

        for year in years:
            page = self.request_page(
                "post",
                "TP/Tziunim_L.aspx",
                params={
                    "src": "",
                    "sys": "tal",
                    "rightmj": 1,
                    "first": "yes",
                    "lang": "",
                },
                data={
                    "__VIEWSTATE": first_page.find("input", {"id": "__VIEWSTATE"})[
                        "value"
                    ],
                    "__VIEWSTATEGENERATOR": first_page.find(
                        "input", {"id": "__VIEWSTATEGENERATOR"}
                    )["value"],
                    "__EVENTVALIDATION": first_page.find(
                        "input", {"id": "__EVENTVALIDATION"}
                    )["value"],
                    "lstSem": str(year - 1) + "9",
                    "old_sem": str(year - 1) + "2",
                    "peula": "",
                    "javas": 1,
                },
            )

            table = page.find("form").find("table")
            for row in table.find_all("tr"):
                cells = row.find_all("td")
                if len(cells) != 13:
                    continue
                cells = [cell.text for cell in cells]

                semester = str(year) + ("a" if cells[0] == "א" else "b")
                course_id = cells[1].replace("-", "")
                grade = "".join([c for c in cells[5] if c.isnumeric()])
                notes = cells[11]

                if grade != "":
                    results.append(
                        GradeInfo(semester, course_id, int(grade), "פטור" in notes)
                    )

        return results

    def get_all_grades(self, years: List[int]) -> List[GradeInfo]:
        result = set()

        study_plan_ids = self.get_study_plan_ids()
        for study_plan_id in study_plan_ids:
            study_plan_grades = self.get_grades(study_plan_id, years)
            result.update(study_plan_grades)

        return list(result)



"""
Download all of the course details at Tel Aviv University.
This file was inspired by https://github.com/TAUHacks/CourseScrape,
and we assume it was created by inspecting the network requests
when accessing https://www.ims.tau.ac.il/Tal/KR/Search_P.aspx.
"""

import json
import sys
import urllib.parse
from dataclasses import dataclass
from enum import Enum
from typing import List, Optional, Tuple

import requests
from bs4 import BeautifulSoup

from tau_tools.logging import log, progress, setup_logging
from tau_tools.utilities import request

HEBREW_SEMESTERS = {"a": "א'", "b": "ב'"}


class Semester(Enum):
    WINTER = ["a"]
    SPRING = ["b"]
    ALL = ["a", "b"]


@dataclass
class ExamInfo:
    moed: str
    date: str
    hour: str
    type: str


@dataclass
class LessonInfo:
    semester: str
    day: str
    time: str
    building: str
    room: str
    type: str


@dataclass
class GroupInfo:
    name: str
    id: str
    group: str
    faculty: str
    lecturer: str
    exams: List[ExamInfo]
    lessons: List[LessonInfo]


def get_schools() -> List[Tuple[str, List[str]]]:
    """
    Returns a list of tuples (school_name, selection_options) of the schools.
    The `selection_options` are either the option for everything in the school,
    or a list of every option for faculties/types in the school.
    """

    search_page = BeautifulSoup(
        request(
            "get",
            "https://www.ims.tau.ac.il/Tal/KR/Search_P.aspx",
            cache_category="courses",
            cache_key="schools",
        ),
        "html.parser",
    )
    all_schools = search_page.select(".table1 select.freeselect.list")
    all_options = []

    for school in all_schools:
        options = [x for x in school.select("option") if x.text != ""]

        if options[0].text.startswith("כל "):
            options = [options[0]]

        all_options.append((school["name"], [option["value"] for option in options]))

    return all_options


def get_exams(
    course_id: str,
    group: str,
    year: str,
    semester: str,
    s: Optional[requests.Session] = None,
) -> List[ExamInfo]:
    """Example: `get_exams("03683087", "01", "2024", "1")`"""

    url = "https://www.ims.tau.ac.il/Tal/KR/Bhina_L.aspx?" + urllib.parse.urlencode(
        {
            "kurs": course_id.replace("-", ""),
            "kv": group,
            "sem": year + semester,
        }
    )
    result_soup = BeautifulSoup(
        request(
            "get",
            url,
            s,
            cache_category="courses",
            cache_key=f"exam-{course_id.replace('-', '')}-{group}-{year}-{semester}",
        ),
        "html.parser",
    )

    if result_soup.select(".msgerrs"):
        # An error ocurred, assume there are no exams
        return []

    header_row, *all_rows = result_soup.select_one(".tableblds").select("tr")

    assert (
        str(header_row)
        == '<tr class="listth"><th>מועד</th><th>תאריך</th><th>שעה</th><th>סוג מטלה</th></tr>'
    )

    result = []
    for row in all_rows:
        cols = row.select("td")
        item = {
            "moed": cols[0].text.strip(),
            "date": cols[1].text.strip(),
            "hour": cols[2].text.strip(),
            "type": cols[3].text.strip(),
        }
        result.append(item)

    return result


def parse_result_page(result_soup: BeautifulSoup, year: str) -> List[GroupInfo]:
    all_rows = result_soup.select_one("#frmgrid table[dir=rtl]").select("tr")
    all_rows = all_rows[1:]
    i = 0
    courses = []
    page_task_id = progress.add_task(
        "[green]Parsing page and fetching course exam data...", total=len(all_rows)
    )
    while i < len(all_rows):
        try:
            if (
                "kotcol" in all_rows[i]["class"]
                and len(list(all_rows[i].children)) == 2
            ):
                # start of course
                i += 1
                # next row is the course name + id
                course_name = list(all_rows[i].children)[1].text
                course_id = next(next(all_rows[i].children).children).strip()
                course_group = (
                    all_rows[i].select_one("span").next_element.next_element.strip()
                )
                i += 1
                # this row has the faculty
                course_faculty = list(all_rows[i].children)[1].text
                i += 1
                # look for the times and instructor section
                while "kotcol" not in all_rows[i - 1]["class"]:
                    i += 1

                course_lecturer = None

                course_lessons: list[LessonInfo] = []

                # store a set of the semesters of the course
                semester_set = set()

                while "רשימת תפוצה" not in all_rows[i].text:
                    split = [x.text.strip() for x in all_rows[i].children]
                    if len(list(all_rows[i].children)) != 7:
                        if course_lecturer is not None:
                            course_lecturer += ", " + split[0]
                        i += 1
                        continue

                    (
                        curr_lecturer,
                        ofen_horaa,
                        building,
                        room,
                        day,
                        time,
                        semester,
                    ) = split

                    if course_lecturer is None and len(curr_lecturer) != 0:
                        course_lecturer = curr_lecturer

                    lesson_info = LessonInfo(
                        semester, day, time, building, room, ofen_horaa
                    )
                    if ofen_horaa != "" and lesson_info not in course_lessons:
                        course_lessons.append(lesson_info)
                        semester_set.add(semester)
                    i += 1

                # look up the exam if listing is only in one semester
                course_exams = []
                if len(semester_set) == 1 and course_lessons[0].semester in [
                    "א'",
                    "ב'",
                ]:
                    sem = ["א'", "ב'"].index(course_lessons[0].semester) + 1
                    try:
                        course_exams = get_exams(
                            course_id, course_group, year, str(sem)
                        )
                    except Exception:
                        pass

                courses.append(
                    GroupInfo(
                        course_name,
                        course_id.replace("-", ""),
                        course_group,
                        course_faculty,
                        course_lecturer,
                        course_exams,
                        course_lessons,
                    )
                )
            else:
                i += 1
        except KeyError:
            i += 1
        progress.update(page_task_id, completed=i)
    progress.update(page_task_id, visible=False)

    return courses


def get_school_courses(
    school_index: int,
    school_details: Tuple[str, List[str]],
    year="2024",
    semester=Semester.ALL,
) -> List[GroupInfo]:
    school_select, school_options = school_details
    result = []

    s = requests.Session()

    payload = {
        "lstYear1": year,
        "txtShemKurs": "",
        "txtShemMore": "",
    }

    if semester == Semester.WINTER:
        payload["ckSem"] = "1"
    elif semester == Semester.SPRING:
        payload["ckSem"] = "2"

    task_id = None
    if len(school_options) != 1:
        task_id = progress.add_task(
            "[blue]Fetching school options...", total=len(school_options)
        )
    for option_index, option in enumerate(school_options):
        data = {**payload, school_select: option}
        page_number = 0
        search_result_page = BeautifulSoup(
            request(
                "post",
                "https://www.ims.tau.ac.il/Tal/KR/Search_L.aspx",
                s,
                data=data,
                headers={
                    "Accept": "text/html,application/xhtml+xml,application/xml",
                    "Content-Type": "application/x-www-form-urlencoded",
                    "User-Agent": "CourseScrape",
                },
                cache_category="courses",
                cache_key=f"courses-{year}-{school_index}-{option_index}-{page_number}",
            ),
            "html.parser",
        )

        while len(search_result_page.select("#next")) > 0:
            page_number += 1
            result += parse_result_page(search_result_page, year)
            log.info(
                f"Finished parsing page {page_number} of school {school_index + 1}"
            )

            data = {"dir1": "1"}
            for hidden_input in search_result_page.select("input[type=hidden]"):
                try:
                    data[hidden_input["name"]] = hidden_input["value"]
                except KeyError:
                    pass

            search_result_page = BeautifulSoup(
                request(
                    "post",
                    "https://www.ims.tau.ac.il/Tal/KR/Search_L.aspx",
                    s,
                    data=data,
                    headers={
                        "Accept": "text/html,application/xhtml+xml,application/xml",
                        "Content-Type": "application/x-www-form-urlencoded",
                        "User-Agent": "CourseScrape",
                    },
                    cache_category="courses",
                    cache_key=f"courses-{year}-{school_index}-{option_index}-{page_number}",
                ),
                "html.parser",
            )

        # The final page
        result += parse_result_page(search_result_page, year)

        if task_id is not None:
            progress.update(task_id, advance=1)

    if task_id is not None:
        progress.update(task_id, visible=False)

    return result


def main(
    output_file_template="courses-{year}{semester}.json",
    year=2024,
    semesters=Semester.ALL,
):
    year = str(year)

    schools = get_schools()
    groups: list[GroupInfo] = []

    with progress:
        school_task = progress.add_task(
            "[purple]Fetching schools...", total=len(schools)
        )
        for school_index, school in enumerate(schools):
            progress.update(school_task, advance=1)
            groups += get_school_courses(school_index, school, year, semesters)

    for semester in semesters.value:
        output_file = output_file_template.format(
            year=str(int(year) + 1), semester=semester
        )

        courses = {}
        for group in groups:
            group_lessons = [
                lesson
                for lesson in group.lessons
                if lesson.semester == HEBREW_SEMESTERS[semester]
            ]
            if len(group_lessons) == 0:
                continue

            if group.id not in courses:
                courses[group.id] = {
                    "name": group.name,
                    "faculty": group.faculty,
                    "exams": group.exams,
                    "groups": [],
                }

            if len(group.exams) != 0:
                courses[group.id]["exams"] = group.exams

            courses[group.id]["groups"].append(
                {
                    "group": group.group,
                    "lecturer": group.lecturer,
                    "lessons": [
                        {k: v for k, v in lesson.__dict__.items() if k != "semester"}
                        for lesson in group_lessons
                    ],
                }
            )

        with open(output_file, "w") as f:
            json.dump(courses, f, ensure_ascii=False)


if __name__ == "__main__":
    setup_logging()

    if len(sys.argv) == 2:
        main(year=int(sys.argv[1]) - 1)
    else:
        main()



"""
Download all of the study plans at Tel Aviv University.
This file has been created by inspecting the network requests when accessing https://exact-sciences.tau.ac.il/search-studies-programs?faculta=0300.
"""

import json
import sys
from dataclasses import dataclass
from typing import Dict, List, Optional

from tau_tools.logging import progress, setup_logging, log
from tau_tools.utilities import request


def request_graphql(
    request_json,
    api_url="https://tochniot.tau.ac.il/graphql",
    delay=1,
    cache_key: Optional[str] = None,
):
    """
    Send a GraphQL request to the `api_url`.
    Check for errors.
    Sleep for `delay` seconds, to prevent overwhelming anything.
    """

    response = request(
        "post",
        api_url,
        json=request_json,
        cache_category="plans",
        cache_key=cache_key,
        delay=delay,
    )
    result = json.loads(response)["data"]

    if (
        "results" in result
        and "body" in result["results"]
        and "Tau" in result["results"]["body"]
        and "errmsgeng" in result["results"]["body"]["Tau"]
    ):
        raise Exception(result["results"]["body"]["Tau"]["errmsgeng"])

    return result


@dataclass
class DepartmentInfo:
    name: str
    id: str


@dataclass
class FacultyInfo:
    name: str
    id: str
    departments: List[DepartmentInfo]


@dataclass
class SchoolInfo:
    name: str
    id: str
    faculties: List[FacultyInfo]


class PlanInfo:
    def __init__(
        self, name: str, id: Optional[str], language: str, year: str, egedid: str
    ):
        self.name = name
        self.id = id
        self.language = language

        if id is None:
            self.find_id(year, egedid)

    def find_id(self, year: str, egedid: str):
        """Tries to find the plan id (tcid) from the `year` and `egedid`"""
        try:
            response = request_graphql(
                {
                    "operationName": "results",
                    "variables": {
                        "filters": {
                            "safa": "1",
                            "shana": year,
                            "egedid": egedid,
                            "tab": "generalExplanation",
                        },
                        "apiUrl": "ydhesberklali",
                    },
                    "query": "query results($apiUrl: String!, $filters: JSON!) {\n  results(apiUrl: $apiUrl, filters: $filters) {\n    body\n    __typename\n  }\n}\n",
                },
                cache_key=f"get-id-{year}-{egedid}",
            )
            self.id = response["results"]["body"][0]["tcid"]
        except Exception:
            pass


def get_schools() -> List[SchoolInfo]:
    """Returns a list of tuples (name, id) of the schools."""

    hierarchy = request_graphql(
        {
            "operationName": "getProgramsHierarchy",
            "variables": {"search": {"safa": "1"}},
            "query": "query getProgramsHierarchy($search: JSON!) {\n  getProgramsHierarchy(search: $search) {\n    id\n    name\n    rama {\n      hasUndefinedBetsefer\n      id\n      name\n      field\n      rama {\n        id\n        name\n        field\n        chug\n        __typename\n      }\n      __typename\n    }\n    __typename\n  }\n}\n",
        },
        cache_key="schools",
    )

    return [
        SchoolInfo(
            school["name"],
            school["id"],
            [
                FacultyInfo(
                    faculty["name"],
                    faculty["id"],
                    [
                        DepartmentInfo(department["name"], department["id"])
                        for department in faculty["rama"]
                    ],
                )
                for faculty in school["rama"]
                if faculty["rama"] is not None
            ],
        )
        for school in hierarchy["getProgramsHierarchy"]
    ]


def get_plans(school: SchoolInfo) -> List[PlanInfo]:
    """Get all of the available plans at the specified school."""

    plans = request_graphql(
        {
            "operationName": "getPrograms",
            "variables": {
                "search": {
                    "safa": "1",
                    "faculta": [
                        {
                            "id": school.id,
                            "name": school.name,
                            "__typename": "HierarchyFilter",
                            "rama": [
                                {
                                    "id": faculty.id,
                                    "name": faculty.name,
                                    "__typename": "BetseferFilter",
                                    "hasUndefinedBetsefer": "false",
                                    "field": "betsefer",
                                    "rama": [
                                        {
                                            "id": department.id,
                                            "name": department.name,
                                            "__typename": "ChugFilter",
                                            "field": "chug",
                                            "chug": True,
                                        }
                                        for department in faculty.departments
                                    ],
                                }
                                for faculty in school.faculties
                            ],
                        }
                    ],
                    "isLoadPrograms": True,
                },
                "from": 0,
                "size": 1000,
            },
            "query": "query getPrograms($search: JSON!, $from: Int, $size: Int) {\n  getPrograms(search: $search, from: $from, size: $size) {\n    total\n    results {\n      tclongkey\n      shana\n      teur\n      toar\n      currentSafa\n      isSafaAfucha\n      tcid\n      egedid\n      faculta\n      teurfaculta\n      teurfacultaeng\n      betsefer\n      teurbetsefer\n      teurbetsefereng\n      maslul\n      chug\n      teurchug\n      teurchugeng\n      teurfaculta2\n      teurfacultaeng2\n      pail\n      title\n      pailheara\n      pailhearaeng\n      showtochnit\n      __typename\n    }\n    from\n    refreshData\n    __typename\n  }\n}\n",
        },
        cache_key=f"plans-{school.id}",
    )

    return [
        PlanInfo(
            result["teur"],
            result["tcid"],
            result["currentSafa"],
            result["shana"],
            result["egedid"],
        )
        for result in plans["getPrograms"]["results"]
    ]


def get_plan(plan: PlanInfo, year=2024) -> Dict[str, List[str]]:
    details = request_graphql(
        {
            "operationName": "results",
            "variables": {
                "filters": {
                    "tcid": plan.id,
                    "shana": str(year),
                    "safa": plan.language,
                    "tab": "programStudy",
                },
                "apiUrl": "ydtochnit",
            },
            "query": "query results($apiUrl: String!, $filters: JSON!) {\n  results(apiUrl: $apiUrl, filters: $filters) {\n    body\n    __typename\n  }\n}\n",
        },
        cache_key=f"plan-{year}-{plan.id}-{plan.language}",
    )

    categories = {}
    courses = set()

    for part in details["results"]["body"][0]["rama"]:
        if "rama" not in part:
            continue

        for category in part["rama"]:
            if "kurs" not in category:
                continue

            category_name = category["teurrama"]
            category_courses = []

            for course in category["kurs"]:
                if course["kursid"] in courses:
                    continue
                courses.add(course["kursid"])

                category_courses.append(course["kursid"])

            if len(category_courses) != 0:
                categories[part["teurrama"] + " - " + category_name] = category_courses

    return categories


def main(output_file_template="plans{year}.json", year=2024):
    result = {}

    schools = get_schools()
    with progress:
        schools_task_id = progress.add_task(
            "[purple]Fetching schools...", total=len(schools)
        )
        for school in schools:
            result[school.name] = {}
            plans = [plan for plan in get_plans(school) if plan.id is not None]
            school_task_id = progress.add_task(
                f"[green]Fetching school '{school.name}' plans...", total=len(plans)
            )
            for plan in plans:
                try:
                    plan_details = get_plan(plan, year)
                    if len(plan_details) != 0:
                        result[school.name][plan.name] = plan_details
                except Exception as e:
                    log.warning(
                        f"Error fetching {plan.name} in {school.name}: [red]{e}[/red]",
                        extra={"markup": True}
                    )
                progress.update(school_task_id, advance=1)
            progress.update(school_task_id, visible=False)
            progress.update(schools_task_id, advance=1)

    with open(output_file_template.format(year=year + 1), "w") as f:
        json.dump(result, f, ensure_ascii=False)


if __name__ == "__main__":
    setup_logging()

    if len(sys.argv) == 2:
        main(year=int(sys.argv[1]) - 1)
    else:
        main()



import os
import time
from typing import Any, Dict, Optional

import requests


def request(
    method: str,
    url: str,
    s: Optional[requests.Session] = None,
    delay=1,
    json: Optional[Any] = None,
    data: Optional[Any] = None,
    headers: Optional[Dict[str, str]] = None,
    cache_category: Optional[str] = None,
    cache_key: Optional[str] = None,
) -> str:
    if cache_category is None:
        cache_file = f"cache/{cache_key}.txt"
    else:
        cache_file = f"cache-{cache_category}/{cache_key}.txt"

    if (
        cache_key is not None
        and os.path.exists(cache_file)
        and not (
            "TAU_TOOLS_FORCE_FETCH" in os.environ
            and len(os.environ["TAU_TOOLS_FORCE_FETCH"]) != 0
        )
    ):
        with open(cache_file, "r") as f:
            response_text = f.read()
        return response_text

    response = (
        s.request(method, url, json=json, data=data, headers=headers)
        if s is not None
        else requests.request(method, url, json=json, data=data, headers=headers)
    )
    time.sleep(delay)

    if cache_key is not None:
        cache_directory = os.path.dirname(os.path.abspath(cache_file))
        if not os.path.isdir(cache_directory):
            os.mkdir(cache_directory)
        with open(cache_file, "w") as f:
            f.write(response.text)

    return response.text



"""
Collects all of the scraped course data into a single `courses.json` which contains only
the names of the courses and the semesters they were taught in.
"""

import json
import os

from tau_tools.logging import log, setup_logging


def main(output_file="courses.json"):
    result = {}

    courses_jsons = [
        f
        for f in sorted(os.listdir("."))[::-1]
        if f.startswith("courses-") and f.endswith(".json")
    ]
    log.info(f"Found courses JSONs: {courses_jsons}")

    for courses_json in courses_jsons:
        semester = courses_json.removeprefix("courses-").removesuffix(".json")

        with open(courses_json, "r") as f:
            courses = json.load(f)

        for course_id in courses:
            if course_id not in result:
                result[course_id] = {
                    "name": courses[course_id]["name"],
                    "semesters": [semester],
                }
            else:
                if result[course_id]["name"] != courses[course_id]["name"]:
                    log.warning(
                        f"Name mismatch of course {course_id}: changed from {courses[course_id]['name']} to {result[course_id]['name']}"
                    )
                result[course_id]["semesters"].append(semester)

    with open(output_file, "w") as f:
        json.dump(result, f, ensure_ascii=False)


if __name__ == "__main__":
    setup_logging()
    main()
